from random import choice
from wtforms.widgets import HiddenInput
from wtforms.widgets.core import HTMLString


TAGS = ('strong', 'em', 'ol', 'ul')


class QuillInput(HiddenInput):
    """
    Renders the text content as hidden input and adds a container for the
    editor.


    """

    def __init__(self, **kwargs):
        tags = list(set(kwargs.pop('tags', TAGS)) & set(TAGS))
        self.placeholders = kwargs.pop('placeholders', {})
        self.placeholder_label = kwargs.pop('placeholder_label', 'Snippets')

        super(QuillInput, self).__init__(**kwargs)

        self.id = ''.join(choice('abcdefghi') for i in range(8))

        self.formats = []
        if 'strong' in tags:
            self.formats.append("'bold'")
        if 'em' in tags:
            self.formats.append("'italic'")
        if 'ol' in tags or 'ul' in tags:
            self.formats.append("'list'")
        if self.placeholders:
            self.formats.append("'placeholder'")

        self.toolbar = []
        if 'strong' in tags:
            self.toolbar.append("'bold'")
        if 'em' in tags:
            self.toolbar.append("'italic'")
        if 'ol' in tags:
            self.toolbar.append("{'list': 'ordered'}")
        if 'ul' in tags:
            self.toolbar.append("{'list': 'bullet'}")
        if self.placeholders:
            options = [f"'{key}'" for key in self.placeholders]
            options = ', '.join(options)
            self.toolbar.append(f"{{'placeholder': [{options}]}}")

    def __call__(self, field, **kwargs):
        input_id = f'quill-input-{self.id}'
        kwargs['id'] = input_id
        input = super(QuillInput, self).__call__(field, **kwargs)

        container_id = f'quill-container-{self.id}'
        formats = ', '.join(self.formats)
        toolbar = ', '.join(self.toolbar)
        placeholders = [
            f"{{id: '{key}', label: '{value}'}}"
            for key, value in self.placeholders.items()
        ]
        placeholders = ', '.join(placeholders)

        return HTMLString(f"""
            <div class="quill-container" id="{container_id}"></div>
            <script>
                window.addEventListener('load', function () {{
                    Quill.register(
                        'modules/placeholder',
                        PlaceholderModule.default(Quill)
                    )
                    var input = document.getElementById('{input_id}');
                    var quill = new Quill('#{container_id}', {{
                        formats: [{formats}],
                        modules: {{
                            toolbar: [{toolbar}],
                            placeholder: {{
                                delimiters: ['', ''],
                                placeholders: [{placeholders}]
                            }}
                        }},
                        theme: 'snow'
                    }});
                    quill.clipboard.dangerouslyPasteHTML(input.value);
                    quill.on('text-change', function() {{
                        input.value = quill.root.innerHTML
                    }});
                    Array.prototype.slice.call(
                        document.querySelectorAll(
                            '.ql-placeholder .ql-picker-item'
                        )
                    ).forEach(item => item.textContent = item.dataset.value);
                    document.querySelector(
                        '.ql-placeholder .ql-picker-label'
                    ).innerHTML = '{self.placeholder_label}';
                }});
            </script>
            {input}
        """)



from onegov.form import Form
from onegov.quill.widgets import QuillInput
from onegov.quill.fields import QuillField


def test_widget_initalization():
    input = QuillInput()
    assert input.formats == ["'bold'", "'italic'", "'list'"]
    assert input.toolbar == [
        "'bold'", "'italic'", "{'list': 'ordered'}", "{'list': 'bullet'}"
    ]

    input = QuillInput(tags=['strong', 'ul'])
    assert input.formats == ["'bold'", "'list'"]
    assert input.toolbar == ["'bold'", "{'list': 'bullet'}"]

    input = QuillInput(tags=['em', 'ol'])
    assert input.formats == ["'italic'", "'list'"]
    assert input.toolbar == ["'italic'", "{'list': 'ordered'}"]

    input = QuillInput(tags=['strong'])
    assert input.formats == ["'bold'"]
    assert input.toolbar == ["'bold'"]

    input = QuillInput(tags=['a', 'i'])
    assert input.formats == []
    assert input.toolbar == []


def test_widget_render():
    form = Form()
    field = QuillField()
    field = field.bind(form, 'html')
    field.data = 'xxx'

    input = QuillInput()
    text = input(field)
    assert f'quill-container-{input.id}' in text
    assert f'quill-input-{input.id}' in text
    assert "['bold', 'italic', 'list']" in text
    assert (
        "['bold', 'italic', {'list': 'ordered'}, {'list': 'bullet'}]"
    ) in text

    input = QuillInput(tags=['em', 'ul'])
    text = input(field)
    assert f'quill-container-{input.id}' in text
    assert f'quill-input-{input.id}' in text
    assert "['italic', 'list']" in text
    assert "['italic', {'list': 'bullet'}]" in text

    input = QuillInput(placeholders={'Hello World': 'Hello, <b>World</b>!'})
    text = input(field)
    assert "{'placeholder': ['Hello World']}" in text
    assert "delimiters: ['', '']" in text
    assert (
        "placeholders: [{id: 'Hello World', label: 'Hello, <b>World</b>!'}]"
        in text
    )



from onegov.form import Form
from onegov.quill.fields import QuillField


def test_field_clean():
    form = Form()

    field = QuillField(tags=[])
    field = field.bind(form, 'html')
    field.data = None
    assert field.validate(form)

    test_data = """
        <h2>
            <span class="">A<strong>B</strong><b>C</b></span>
        </h2>
        <p>
            <span class="md-line md-end-block">
                <span class=""><em>D</em><i>E</i></span><a href="xx">F</a>
                <br>
            </span>
            <span class="md-line md-end-block">
                <script>XXXX</script>
            </span>
        </p>
            <ul>
                <li>1</li>
                <li>2</li>
                <li>3</li>
            </ul>
            <ol>
                <li>1</li>
                <li>2</li>
                <li>3</li>
            </ol>
    """

    field = QuillField(tags=[])
    field = field.bind(form, 'html')
    field.data = test_data
    assert field.validate(form)
    assert field.data.replace(' ', '').replace('\n', '') == (
        'ABC<p>DEF<br>XXXX</p>123123'
    )

    field = QuillField(tags=['strong'])
    field = field.bind(form, 'html')
    field.data = test_data
    assert field.validate(form)
    assert field.data.replace(' ', '').replace('\n', '') == (
        'A<strong>B</strong>C<p>DEF<br>XXXX</p>123123'
    )

    field = QuillField(tags=['em', 'ol', 'ul'])
    field = field.bind(form, 'html')
    field.data = test_data
    assert field.validate(form)
    assert field.data.replace(' ', '').replace('\n', '') == (
        'ABC<p><em>D</em>EF<br>XXXX</p>'
        '<ul><li>1</li><li>2</li><li>3</li></ul>'
        '<ol><li>1</li><li>2</li><li>3</li></ol>'
    )

    field = QuillField(tags=['ol'])
    field = field.bind(form, 'html')
    field.data = test_data
    assert field.validate(form)
    assert field.data.replace(' ', '').replace('\n', '') == (
        'ABC<p>DEF<br>XXXX</p>'
        '<li>1</li><li>2</li><li>3</li>'
        '<ol><li>1</li><li>2</li><li>3</li></ol>'
    )

    field = QuillField(tags=['strong', 'ul'])
    field = field.bind(form, 'html')
    field.data = test_data
    assert field.validate(form)
    assert field.data.replace(' ', '').replace('\n', '') == (
        'A<strong>B</strong>C<p>DEF<br>XXXX</p>'
        '<ul><li>1</li><li>2</li><li>3</li></ul>'
        '<li>1</li><li>2</li><li>3</li>'
    )

    field = QuillField()
    field = field.bind(form, 'html')
    field.data = test_data
    assert field.validate(form)
    assert field.data.replace(' ', '').replace('\n', '') == (
        'A<strong>B</strong>C<p><em>D</em>EF<br>XXXX</p>'
        '<ul><li>1</li><li>2</li><li>3</li></ul>'
        '<ol><li>1</li><li>2</li><li>3</li></ol>'
    )

    field = QuillField(tags=['a', 'i', 'b'])
    field = field.bind(form, 'html')
    field.data = test_data
    assert field.validate(form)
    assert field.data.replace(' ', '').replace('\n', '') == (
        'ABC<p>DEF<br>XXXX</p>123123'
    )
